import React, { useRef, useEffect, useState, useImperativeHandle } from 'react';
import { Animated } from 'react-native';
import { faEye, faEyeSlash } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-native-fontawesome';
import { useField } from '@unform/core';
import { OnlyNumbers } from '../../../services';
import { maskValue } from '../../MaskText/mask.utils';
import { IPropsInput } from '../input.interface';
import {
   Container,
   InputContainer,
   InputStyle,
   ViewPassword,
   TitleText,
   Prefix,
   Suffix,
   TextError,
} from './Input.style';

export const Input = React.forwardRef<any, IPropsInput>(function Input(props: IPropsInput, ref: any) {
   props = { decimals: 0, ...props };

   const {
      name,
      title,
      textRight = false,
      textCenter = false,
      prefix,
      decimals = 0,
      theme: themeInput,
      suffix,
      loading = false,
      onEnter = () => {},
      number = false,
      withoutDotNumber = false,
      CEP = false,
      CNPJ = false,
      CPF = false,
      RG = false,
      phone = false,
      noSpaces = false,
      ...rest
   } = props;

   const inputRef = useRef<any>();

   useImperativeHandle(ref, () => inputRef.current, [inputRef]);

   // Unform
   const { registerField, defaultValue = '', error, fieldName } = useField(name);
   const defaultValueInput = maskValue(defaultValue, props);

   // Se Input está em focus
   const [focus, setFocus] = useState(false);
   const [value, setValue] = useState(defaultValueInput);

   // onEnter
   // useEventListener('keydown.enter', () => onEnter(), inputRef);

   // Event: Input
   // useEventListener(
   //    'input',
   //    (evt) => {
   //       inputRef.current.value = maskValue(inputRef.current.value, props);
   //    },
   //    inputRef,
   // );

   useEffect(() => {
      registerField({
         name: fieldName,
         ref: inputRef.current,
         path: 'value' as any,
         setValue(_ref: any, value: string) {
            inputRef.current.value = maskValue(value, props);
         },
         getValue(_ref: any) {
            if (number) return OnlyNumbers(_ref.value);
            if (CEP) return OnlyNumbers(_ref.value);
            return _ref.value;
         },
      });
   }, [fieldName, registerField]);

   const forceShowTitle = false;
   const [hasText, setHasText] = useState(false);
   const animateTitle = useRef(new Animated.Value(forceShowTitle ? 1 : 0)).current;

   useEffect(() => {
      if (!forceShowTitle) {
         if (focus) {
            Animated.timing(animateTitle, {
               toValue: 1,
               duration: 150,
               useNativeDriver: true,
            }).start();
         } else if (!hasText) {
            Animated.timing(animateTitle, {
               toValue: 0,
               duration: 150,
               useNativeDriver: true,
            }).start();
         }
      }
   }, [focus]);

   return (
      <Container>
         {title && (
            <Animated.View
               style={{
                  opacity: animateTitle,
                  transform: [
                     {
                        translateY: animateTitle.interpolate({
                           inputRange: [0, 1],
                           outputRange: [25, 0],
                        }),
                     },
                  ],
                  zIndex: focus ? 1 : 0,
               }}
            >
               <TitleText>{title}</TitleText>
            </Animated.View>
         )}

         <InputContainer focus={focus} error={!!error}>
            {prefix && <Prefix>{prefix}</Prefix>}
            <InputStyle
               themeInput={themeInput}
               ref={inputRef}
               keyboardAppearance="dark"
               defaultValue={defaultValueInput}
               placeholder={focus ? '' : title}
               keyboardType={number ? 'numeric' : 'default'}
               // secureTextEntry={password && hidePassword}
               onChangeText={(_value: string) => {
                  setHasText(!!_value);
                  if (inputRef.current) {
                     inputRef.current.value = maskValue(_value, props);
                  }
                  setValue(maskValue(_value, props));
               }}
               onFocus={() => setFocus(true)}
               onBlur={() => setFocus(false)}
               prefix={!!prefix}
               suffix={!!suffix}
               textRight={textRight}
               textCenter={textCenter}
               value={value}
            />
            {suffix && <Suffix>{suffix}</Suffix>}
         </InputContainer>

         {/* {password && ( */}
         {/*   <ViewPassword activeOpacity={0.8} onPress={() => setHidePassword(!hidePassword)}> */}
         {/*      {hidePassword && <FontAwesomeIcon icon={faEyeSlash} size={20} />} */}
         {/*      {!hidePassword && <FontAwesomeIcon icon={faEye} size={20} />} */}
         {/*   </ViewPassword> */}
         {/* )} */}

         {!!error && <TextError>{error}</TextError>}
      </Container>
   );
});
